from .base import Resource


class Ping(Resource):
    key = "ping"

    def ping_service(self):
        """
        Ping the server to verify it is reachable

        :return:  A ``str`` object.  A successful ping will return the string 'pong'.
        """

        endpoint = ""
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters({}, {})
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse



import json
from .base import Resource


class Intelliseeds(Resource):
    key = "intelliseed"

    param_model = {
        'childAccountId',
        'percentOfList',
        'regions',
        'simulatedEngagementOption',
        'type'
    }

    def get_intelliseeds(self, **kwargs):
        """
        Get an updated IntelliSeed list

        :param int childAccountId:  The child account to narrow results
        :param str type:  The type of IntelliSeed list to pull.

            Accepts:  ``PUBLIC``, ``PRIVATE``, ``EXCLUSIVE``. Default is ``PRIVATE``
        :param int percentOfList:  The integer percent, 1-100, of the IntelliSeed list to pull. Default is 100
        :param str simulatedEngagementOption:  Which engagement types of IntelliSeed accounts to include.

            Accepts:  ``ALL``, ``ENGAGING``, and ``NON_ENGAGING``.  Default is ``ALL``

        :param regions:  List of strings.  The regions to include IntelliSeed accounts from.
            For options, See
            `Current Regions <https://inboxtracker.readthedocs.io/en/latest/resources/regions.html#supported-regions>`_

            Default is all regions.

        :return:  A ``list`` object of ``dict`` containing details of an IntelliSeed address, including email address,
            location, and ISP.
        """

        endpoint = ""
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse

    def get_last_update(self, **kwargs):
        """
        Get the time of the last update to IntelliSeed List

        :param int childAccountId:  The child account to narrow results

        :return:  A ``list`` object of 'dict' containing the account ID and last update date of the IntelliSeed list.
        """

        endpoint = "/lastUpdate"
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse

    def get_filter_sets(self, **kwargs):
        """
        Get the list of IntelliSeed filter sets configured for an account

        :param int childAccountId: The child account to narrow results

        :return: A ``list`` object of 'dict' containing details of a filter set, such as the name of the IntelliSeed
            list.
        """

        endpoint = "/filter_sets"
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse

    def get_intelliseeds_filtered(self, filterSetId, **kwargs):
        """
        Gets the latest IntelliSeed list associated with the filter set id provided

        :param int filterSetId: The id of the IntelliSeed filter set
        :param int childAccountId: The child account to narrow results

        :return: A ``list`` object of 'dict' containing details of the specified filter set, such as the name and ID of
            the IntelliSeed list.
        """

        endpoint = "/" + str(filterSetId)
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse

    def create_intelliseed_filter(self, **kwargs):
        """
        Places a new IntelliSeed filter set under the given account

        :param str name: Name of the filter set
        :param str listType: Privacy level of list.

            Accepts: ``PUBLIC``, ``EXCLUSIVE``, ``PRIVATE``
        :param str simulatedEngagementOption: Engagement level of the IntelliSeeds to include in the list.

            Accepts: ``ALL``, ``ENGAGING``, ``NON_ENGAGING``
        :param int percentOfList: Percent of the list to include. Integer from 0-100.
        :param regions: List of strings.  The regions to include.  For options, See
            `Current Regions <https://inboxtracker.readthedocs.io/en/latest/resources/regions.html#supported-regions>`_
        :param int childAccountId: The child account to place filter

        :return: A ``dict`` object containing the details of the crated IntelliSeed list, such as the name and ID.
        """

        data_model = {
            'name': 'name',
            'listType': 'listType',
            'simulatedEngagementOption': 'simulatedEngagementOption',
            'percentOfList': 'percentOfList',
            'regions': 'regions'
        }

        endpoint = "/filter_sets"
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        data = self.translate_keys(kwargs, data_model)
        data = json.dumps(data)
        apiResponse = self.request("POST", apiUrl, params=parameters, data=data)

        return apiResponse

    def delete_intelliseed_filter(self, filterSetId, **kwargs):
        """
        Removes the IntelliSeed filter set using the filter set id provided

        :param int filterSetId: The id of the IntelliSeed filter set
        :param int childAccountId: The child account to narrow results

        :return: A ``dict`` object containing the details of the deleted IntelliSeed list, such as the name and ID.
        """

        endpoint = "/filter_sets/" + str(filterSetId)
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("DELETE", apiUrl, params=parameters)

        return apiResponse


import responses
from inboxtracker import InboxTracker


@responses.activate
def test_get_last_update():
    it = InboxTracker("fake-key")
    endpoint = "/lastUpdate"
    url = it.seeds.uri + endpoint
    responses.add(
        responses.GET,
        url,
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    json_output = it.seeds.get_last_update()
    assert json_output == 'yay'


@responses.activate
def test_get_usage():
    it = InboxTracker("fake-key")
    endpoint = "/usage"
    url = it.seeds.uri + endpoint
    responses.add(
        responses.GET,
        url,
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    json_output = it.seeds.get_usage()
    assert json_output == 'yay'


@responses.activate
def test_get_seeds():
    it = InboxTracker("fake-key")
    endpoint = ""
    url = it.seeds.uri + endpoint
    responses.add(
        responses.GET,
        url,
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    json_output = it.seeds.get_seeds()
    assert json_output == 'yay'



import copy
import requests
import inboxtracker
from .exceptions import InboxTrackerAPIException
import base64


class RequestsTransport(object):
    def __init__(self):
        import requests
        self.sess = requests.Session()

    def request(self, method, uri, params, **kwargs):
        response = self.sess.request(method, uri, params=params, **kwargs)

        if response.status_code == 204:
            return True
        if not response.ok:
            raise InboxTrackerAPIException(response)
        try:
            if 'results' in response.json():
                return response.json()['results']
        except:
            return response.text
        return response.json()


class Resource(object):
    key = ""

    def __init__(self, base_uri, api_key, transport_class=RequestsTransport):
        self.base_uri = base_uri
        self.api_key = api_key
        self.transport = transport_class()

    @property
    def uri(self):
        return "%s/%s" % (self.base_uri, self.key)

    def request(self, method, uri, params, **kwargs):
        key_b64 = base64.b64encode(bytes(self.api_key, "ascii"))
        headers = {
            'User-Agent': 'python-inboxtracker/' + inboxtracker.__version__,
            'Content-Type': 'application/json',
            "Authorization": "Basic " + key_b64.decode("ascii")
        }
        response = self.transport.request(method, uri, params=params, headers=headers,
                                          **kwargs)
        return response

    def SetParameters(self, args, model_remap):
        parameters = {}

        model = copy.deepcopy(args)

        # Intersection of keys that need to be remapped
        data_remap_keys = set(model_remap).intersection(model.keys())

        for k in data_remap_keys:
            parameters[k] = model[k]

        return parameters

    def translate_keys(self, args, model_remap):
        model = copy.deepcopy(args)
        # Intersection of keys that need to be remapped
        model_remap_keys = frozenset(model_remap.keys())
        data_remap_keys = model_remap_keys.intersection(model.keys())

        for from_key in data_remap_keys:
            # Remap model keys to match API
            if from_key in model:
                to_model = model
                to_key = model_remap[from_key]
                if '/' in to_key:
                    # Nested within a dict
                    into_list = to_key.split('/')
                    to_key = into_list[-1]
                    to_model = model.setdefault(into_list[0], {})

                # Move from current key and place into new key
                to_model[to_key] = model.pop(from_key)

        return model

    def get(self):
        raise NotImplementedError

    def list(self):
        raise NotImplementedError

    def create(self):
        raise NotImplementedError

    def update(self):
        raise NotImplementedError

    def delete(self):
        raise NotImplementedError



from .base import Resource


class Domains(Resource):
    key = "domains"

    param_model = {
        'childAccountId'
    }

    def get_available_domains(self, **kwargs):
        """
        Get all domains available within your account

        :param int childAccountId:  The child account to narrow results

        :return:  A ``list`` object of ``dict`` containing details of the domains available.
        """

        endpoint = "/available"
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse




from .base import Resource


class Regions(Resource):
    key = "regions"

    param_model = {
    }

    def get_regions(self):
        """
        Gets current list of regions supported by the system.

        :return: A ``list`` object of available regions as ``str``.  See
            `Current Regions <https://inboxtracker.readthedocs.io/en/latest/resources/regions.html#supported-regions>`_
            for an updated list of supported regions.
        """

        endpoint = ""
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters({}, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse


from .base import Resource


class Campaigns(Resource):
    key = "campaigns"

    param_model = {
        'body',
        'campaignIdentifier',
        'childAccountId',
        'domain',
        'embed',
        'fromAddress',
        'headerKey',
        'headerValue',
        'ipAddress',
        'limitToSpecifiedDomains',
        'order',
        'page',
        'per_page',
        'qd',
        'subject'
    }

    def get_campaigns(self, **kwargs):
        """
        Get all campaign details for a given time period

        :param str qd:  A date range query parameter.

            Accepts:  ``since:YYYYMMDD``, ``between:YYYYMMDDhhmmss,YYYYMMDDhhmmss``, and ``daysBack:N``.

            Examples:  ``since:20190601``, ``between:20191001000000,20191002060000``, ``daysBack:30``

        :param str campaignIdentifier:  The campaign identifier value to search for.
        :param str fromAddress:  The from address to search for
        :param int childAccountId:  The child account to narrow results
        :param domain:  List of strings.  Narrow results to certain domains that are under this API key
            (multiple allowed)
        :param str headerKey:  Campaigns that used a specific header key
        :param str headerValue:  Campaigns that used a specific header value
        :param str subject:  Subject search criteria
        :param str body:  Body search criteria
        :param int page:  The page to query for in pagination
        :param int per_page:  The amount of records per page you wish to query for (max 100)
        :param str order:  The property to sort by ('property' for decending, '-property' for ascending)

            Accepts:  ``firstSeen``, ``-firstSeen``, ``lastSeen``, ``-lastSeen``, ``inbox``, ``-inbox``, ``spam``,
            ``-spam``
        :param str embed:  The objects within the return model you wish to embed.

            Accepts:  ``sendingIps``, ``gmailCategories``, ``authInbox``

        :return:  A ``list`` object of ``dict`` containing details for a campaign.
        """

        endpoint = ""
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse

    def get_campaign_by_id(self, campaignId, **kwargs):
        """
        Get campaign details by querying campaign identifier.

        :param int campaignId: Integer campaign identifier
        :param int childAccountId:  The child account to narrow results
        :param str embed:  The objects within the return model you wish to embed.

            Accepts:  ``sendingIps``, ``gmailCategories``, ``authInbox``

        :return:  A ``dict`` object containing details of the specified campaign.
        """

        endpoint = "/" + str(campaignId)
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse

    def get_ip_stats(self, **kwargs):
        """
        Get statistics for all IP addresses in your account

        :param str qd:  A date range query parameter.

            Accepts:  ``since:YYYYMMDD``, ``between:YYYYMMDDhhmmss,YYYYMMDDhhmmss``, and ``daysBack:N``.

            Examples:  ``since:20190601``, ``between:20191001000000,20191002060000``, ``daysBack:30``
        :param childAccountId:  List of int.  The child account(s) to narrow results
        :param ipAddress:  List of strings.  The ip address to narrow results to (multiple allowed)
        :param domain:  List of strings.  Narrow results to certain domains that are under this API key (multiple
            allowed)
        :param bool limitToSpecifiedDomains:  Only include data for the supplied domains (vs all domains using the IPs)
        :param int page:  The page to query for in pagination
        :param int per_page:  The amount of records per page you wish to query for (max 100)

        :return:  A ``list`` object of ``dict`` containing campaign details for each IP address.
        """

        endpoint = "/sendingIps"
        apiUrl = self.uri + endpoint
        parameters = self.SetParameters(kwargs, self.param_model)
        apiResponse = self.request("GET", apiUrl, params=parameters)

        return apiResponse



import responses
from inboxtracker import InboxTracker


@responses.activate
def test_get_campaigns():
    it = InboxTracker("fake-key")
    qd = "test-qd"
    endpoint = ""
    url = it.campaigns.uri + endpoint
    responses.add(
        responses.GET,
        url,
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    json_output = it.campaigns.get_campaigns(qd=qd)
    assert json_output == 'yay'

@responses.activate
def test_get_campaign_by_id():
    it = InboxTracker("fake-key")
    campaignId = "test-campaign"
    embed = "test-embed"
    endpoint = "/" + str(campaignId)
    url = it.campaigns.uri + endpoint
    responses.add(
        responses.GET,
        url,
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    json_output = it.campaigns.get_campaign_by_id(campaignId, embed=embed)
    assert json_output == 'yay'

@responses.activate
def test_get_ip_stats():
    it = InboxTracker("fake-key")
    qd = "test-qd"
    endpoint = "/sendingIps"
    url = it.campaigns.uri + endpoint
    responses.add(
        responses.GET,
        url,
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    json_output = it.campaigns.get_ip_stats(qd=qd)
    assert json_output == 'yay'
